#################################################################################
#                                                                               #
#   2023 Giacomo Riccardo Hvaring                                               #
#                                                                               #
#   Generation of a signal composed of equally spaced tones sent to the SDR     #
#   for a specified total duration.                                             #
#                                                                               #
#################################################################################

from . import uhd_lib
import numpy as np
from uhd.libpyuhd.types import time_spec

MHz = 1e6


import time

def main(usrp, duration, sampling_rate, frequencies, amplitudes, phases):

    resolution = 0.01 * MHz
    final_length = 10 * int(sampling_rate / resolution)
    n_frequencies = len(frequencies)
    if n_frequencies == 1:
        period_size = 100
    else:
        center = n_frequencies // 2
        min_frequency = - frequencies[center-1]
        period_size = round(sampling_rate / min_frequency)
    times = np.arange(2 * period_size) / sampling_rate
    signal = np.zeros(times.size, dtype=np.complex64)

    #print(f'Period: {period_size} points')

    # time_now = usrp.set_time_now(time_spec(0.1))
    # time_now = usrp.get_time_now()
    # print(f'Time now: {time_now}')
    # usrp.clear_command_time()

    for a, f, p in zip(amplitudes, frequencies, phases):
        signal += a * uhd_lib.single_tone(times, f, p)

    signal = uhd_lib.fill_buffer(signal, final_length)

    n_points_to_send = int(duration * sampling_rate)
    streamer = uhd_lib.get_streamer(usrp)
    
    sent = uhd_lib.send(signal, n_points_to_send, streamer)

    if sent != n_points_to_send:
        raise ValueError(f"Sent {sent} sample points instead of {n_points_to_send}.")
    

if __name__ == '__main__':
    main()